/*
    Clay Shooter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Clay Shooter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Clay Shooter.  If not, see <http://www.gnu.org/licenses/>.

#-----------------------------------------------------------------------------------------#

 Author: Federico Salotti
 Organization/Company: Mr_Robot_Computers_Repair
 Country: Italy Barga(LU)
 release: 1.0 (08 March 2020 14:02)
 website: http://mrrobotcomputers.hopto.org

*/


#include <SFML/Graphics.hpp>
#include <iostream>
#include <time.h>
#include <cstring>
#include <fstream>
#include "symbols.h"
#include "obj_status.h"
#include "plate.h"
#include "pieces.h"
#include "particles.h"
#include "plateBattery.h"
#include "messageBox.h"


sf::Clock chrono;
sf::Clock animation_chrono;
sf::Clock shoot_chrono;
sf::Clock chrono_horizon;
sf::Clock chrono_clean;
sf::Sprite view;
sf::Mouse mouse;


bool shoot=false;
bool welcome=true;
bool win=false;
bool game_over=false;
bool pause=false;
bool isRestart=true;

unsigned int lv=1;
float plateSpawnDelay=1.5;
float plateHorizionSpeed=1.0;
float cleanDelay=1.0;

sf::Color sky;

void next_round(int argc, char** argv,plateBattery &plate_battery,sf::Texture &texture, sf::Texture &texture2, ObjStatus &bullet)
{
	lv+=1;
	plate_battery.newPlates(texture,texture2);
	plate_battery.assignPlatesSpawn();
	plate_battery.assignPlatesMovement(lv);
	plate_battery.resetParticles();
	if(argc > 1)
	{
		if(strcmp(argv[1],"medium") == 0)bullet.set(15);
		else if(strcmp(argv[1],"hard") == 0)bullet.set(12);
		else bullet.set(18);
	}
	else bullet.set(18);
	bullet.setColor(sf::Color::White);
	bullet.setBlink(false);
	if(plateHorizionSpeed > 0.5) plateHorizionSpeed-=0.2;
}

void restart(int argc, char **argv,plateBattery &plate_battery,sf::Texture &texture, sf::Texture &texture2,ObjStatus &bullet, ObjStatus &info_missing,ObjStatus &info_score)
{
	lv=1;
	plate_battery.reset(texture,texture2,lv);
	plate_battery.resetParticles();
	info_missing.setColor(sf::Color::White);
	info_missing.setBlink(false);
	info_missing.set(0);
	if(argc > 1)
	{
		if(strcmp(argv[1],"medium") == 0)bullet.set(15);
		else if(strcmp(argv[1],"hard") == 0)bullet.set(12);
		else bullet.set(18);
	}
	else bullet.set(18);
	bullet.setColor(sf::Color::White);
	bullet.setBlink(false);

	info_score.set(0);
	info_score.setBlink(false);
	info_score.setColor(sf::Color::White);
	welcome=false;
	game_over=false;
	plateHorizionSpeed=1.0;
	isRestart=true;
}

int hscore(bool write=false, int value=0)
{
	if(!write)
	{
		std::ifstream reader("hscore");
		std::string score;

		std::getline(reader,score);
		reader.close();
		return stoi(score);
	}
	else
	{
		std::ofstream writer("hscore");
		writer << value;
		writer.close();

		return 1;
	}
}

int main(int argc, char** argv)
{
	srand(time(nullptr));

	sf::RenderWindow window(sf::VideoMode(1024,576),"CLAY SHOOTER");
	sf::ContextSettings settings;
	settings.antialiasingLevel=8;

	window.setFramerateLimit(0);
	window.setVerticalSyncEnabled(true);
	window.setMouseCursorVisible(false);

	sf::Font font;
	font.loadFromFile("font/Px437_PhoenixEGA_8x8.ttf");

	sf::Texture playerfront_texture;
	playerfront_texture.loadFromFile("player_front.png");

	sf::Texture playerdx_texture;
	playerdx_texture.loadFromFile("player_dx.png");

	sf::Texture playersx_texture;
	playersx_texture.loadFromFile("player_sx.png");

	sf::Sprite player;
	player.setTexture(playerfront_texture);
	player.setPosition(480,500);

	sf::Texture plate_texture;
	plate_texture.loadFromFile("plate.png");

	sf::Texture missing_plate_texture;
	missing_plate_texture.loadFromFile("missing_plate.png");

	sf::Texture lv_ico;
	lv_ico.loadFromFile("lv.png");

	sf::Texture plate_broke;
	plate_broke.loadFromFile("break.png");

	sf::Texture texture_bg;
	texture_bg.loadFromFile("montagna.png");

	sf::Texture bullet_texture;
	bullet_texture.loadFromFile("proiettile.png");

	sf::Sprite bg;
	bg.setTexture(texture_bg);

	plateBattery plate_battery(plate_texture,plate_broke);
	plate_battery.assignPlatesSpawn();
	plate_battery.assignPlatesMovement(lv);

	sf::Texture view_texture;
	view_texture.loadFromFile("view.png");
	view.setTexture(view_texture);

	messageBox start_box(font,"CLAY SHOOTER","press space to start!");
	messageBox win_box(font,"WIN!","ready LV "+std::to_string(lv));
	messageBox pause_box(font,"Pause","Press R to resume");
	messageBox game_over_box(font,"Game Over","press space to restart!");

	ObjStatus info_plates(plate_texture,font,plate_battery.getPlatesNumber());
	info_plates.setPosition(50,420);

	ObjStatus info_bullet(bullet_texture,font,0);
	if(argc > 1 )
	{
		if(strcmp(argv[1],"medium") == 0) info_bullet.set(15);
		if(strcmp(argv[1],"hard") == 0) info_bullet.set(12);
	}
	else info_bullet.set(18);

	info_bullet.setPosition(50,460);
	info_bullet.scale(2);

	ObjStatus info_missing(missing_plate_texture,font,0);
	info_missing.setPosition(200,420);

	ObjStatus info_lv("LV",font,lv);
	info_lv.setPosition(200,460);

	ObjStatus info_score("SCORE",font,0);
	info_score.setPosition(750,420);

	ObjStatus info_hscore("H-SCORE",font,0);
	info_hscore.setColor(sf::Color::Yellow);
	info_hscore.setBlink();
	info_hscore.setPosition(750,450);

	if(argv[1])
	{
		if(strcmp(argv[1],"medium") == 0)
		{
			plateSpawnDelay=1.25;
			info_bullet.set(15);
		}

		if(strcmp(argv[1],"hard") == 0)
		{
			plateSpawnDelay=1.0;
			info_bullet.set(12);
		}
	}

	ObjStatus info_mode("",font,0);

	if(argc > 1)
	{
		if(strcmp(argv[1],"medium") == 0 || strcmp(argv[1],"hard") == 0)
			info_mode.setText("MODE "+(std::string)argv[1]);

		else info_mode.setText("MODE easy");
	}
	else
		info_mode.setText("MODE easy");

	info_mode.setPosition(750,480);
	info_mode.hideValue();
	info_mode.setColor(sf::Color(0x94,0x33,0xd3));


	while(window.isOpen())
	{
		sf::Event event;

		if(info_missing.getValue() == 2) info_missing.setColor(sf::Color(0xf0,0x5e,0x23));
		if(info_missing.getValue() == 3) {info_missing.setColor(sf::Color::Red);info_missing.setBlink();}
		if(info_missing.getValue() > 3) game_over=true;
		if(info_score.getValue() > info_hscore.getValue())
		{
			info_hscore.setBlink(false);
			info_hscore.setColor(sf::Color::Green);
			info_score.setColor(sf::Color::Red);
			info_score.setBlink();
		}

		if(info_bullet.getValue() <= 3  && info_bullet.getValue() > 0) info_bullet.setColor(sf::Color::Red);
		if(info_bullet.getValue() == 0) info_bullet.setBlink();

		if(!game_over && !welcome)
		{
			if(isRestart)
			{
				info_hscore.set(hscore());
				isRestart=false;
			}

			if(!pause)
			{

				if(plate_battery.isEmpty()) win=true;
				if(win) next_round(argc,argv,plate_battery,plate_texture,plate_broke,info_bullet);
				win_box.updateMsg("WIN!","ready LV"+std::to_string(lv));
				info_plates.update(plate_battery.getPlatesNumber());
				info_lv.update(lv);


				if(chrono.getElapsedTime().asSeconds() > plateSpawnDelay)
				{
					plate_battery.preparePlate();
					plate_battery.check_missing(info_missing);
					plate_battery.clean(info_score);
					chrono.restart();
				}

				if(chrono_horizon.getElapsedTime().asSeconds() > plateHorizionSpeed)
				{
					plate_battery.checkSteppingAway(info_missing);
					chrono_horizon.restart();
				}

				plate_battery.movePlates();
				plate_battery.moveParticles();

				if(mouse.getPosition(window).y > 0)view.setPosition(mouse.getPosition(window).x,mouse.getPosition(window).y);
				if(mouse.getPosition(window).x < 400)player.setTexture(playersx_texture);
				else if(mouse.getPosition(window).x > 600)player.setTexture(playerdx_texture);
				else player.setTexture(playerfront_texture);
			}

			while(window.pollEvent(event))
			{
				if(event.type == sf::Event::Closed) window.close();

				else if(sf::Keyboard::isKeyPressed(sf::Keyboard::P))
					pause=true;

				else if(sf::Keyboard::isKeyPressed(sf::Keyboard::R))
					pause=false;

				else if(sf::Mouse::isButtonPressed(sf::Mouse::Left))
				{
					if(shoot_chrono.getElapsedTime().asSeconds() > SHOOT_DELAY)
					{
						if(info_bullet.getValue() != 0)
						{
							plate_battery.check_collision(view);
							shoot=true;
							info_bullet.sub();
						}

						shoot_chrono.restart();
					}
				}

			}

			if(shoot)
				sky=sf::Color::White;

			else
			{
				if(lv% 3 == 1)sky=sf::Color::Cyan;
				else if(lv%3 == 2) sky=sf::Color(0xff,0xa5,0x00);
				else if(lv%3 == 0) sky=sf::Color(0x00,0x00,0x33);
				else sky=sf::Color::Cyan;
			}

			if(animation_chrono.getElapsedTime().asSeconds() > 0.1)
			{
				shoot=false;
				animation_chrono.restart();
			}

			window.clear(sky);
			window.draw(bg);
			info_bullet.draw(window);
			info_plates.draw(window);
			info_missing.draw(window);
			info_score.draw(window);
			info_hscore.draw(window);
			info_mode.draw(window);
			info_lv.draw(window);
			window.draw(player);
			plate_battery.draw(window);
			window.draw(view);
			if(win)
			{
				while(chrono.getElapsedTime().asSeconds() < 2)
				{
					window.clear(sky);
					window.draw(bg);
					win_box.draw(window);
					window.display();
				}
				win=false;
			}
			if(pause) pause_box.draw(window);
			window.display();
		}
	else
	{
		while(window.pollEvent(event))
		{
			if(event.type == sf::Event::Closed) window.close();
			if(sf::Keyboard::isKeyPressed(sf::Keyboard::Space))
			{
				if(info_score.getValue() > hscore()) hscore(true,info_score.getValue());
				restart(argc,argv,plate_battery,plate_texture,plate_broke,info_bullet,info_missing,info_score);

			}

		}

		window.clear();
		window.draw(bg);
		if(welcome) start_box.draw(window);
		else game_over_box.draw(window);
		window.display();
	}

}

	return 0;
}

