/*
    Clay Shooter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Clay Shooter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Clay Shooter.  If not, see <http://www.gnu.org/licenses/>.

#-----------------------------------------------------------------------------------------#

 Author: Federico Salotti
 Organization/Company: Mr_Robot_Computers_Repair
 Country: Italy Barga(LU)
 release: 1.0 (08 March 2020 14:02)
 website: http://mrrobotcomputers.hopto.org

*/


class ObjStatus
{
public:
	ObjStatus(sf::Texture &texture,sf::Font &font,int n, short charSize=16)
	{
		this->_ico.setTexture(texture);
		this->_n=n;

		this->_q_ui.setFont(font);
		this->_q_ui.setCharacterSize(charSize);
		this->_q_ui.setString("x"+std::to_string(this->_n));
		this->_q_ui.setFillColor(sf::Color::White);
	}

	ObjStatus(std::string title,sf::Font &font,int n,short charSize=16)
	{
		this->_text=title;


		this->_title_ui.setString(this->_text);
		this->_title_ui.setFont(font);
		this->_title_ui.setCharacterSize(charSize);
		this->_title_ui.setFillColor(sf::Color::White);

		this->_q_ui.setFont(font);
		this->_q_ui.setCharacterSize(16);
		this->_q_ui.setString(std::to_string(this->_n));
		this->_q_ui.setFillColor(sf::Color::White);

		this->_textual=true;
	}

	void setText(std::string text)
	{
		this->_text=text;
		this->_title_ui.setString(this->_text);
	}

	int getValue()
	{
		return this->_n;
	}

	void set(int n)
	{
		this->_n=n;
	}

	void sub(int n=1)
	{
		this->_n-=n;
	}

	void add(int n=1)
	{
		this->_n+=n;
	}

	void setPosition(int x, int y)
	{
		if(!this->_textual)
		{
			this->_ico.setPosition(x,y);
			this->_q_ui.setPosition(x+35,y);
		}
		else
		{
			this->_title_ui.setPosition(x,y);
			this->_q_ui.setPosition(x+this->_text.length()*this->_title_ui.getCharacterSize(),y);
		}
	}

	void scale(int n)
	{
		this->_ico.setScale(this->_ico.getScale().x/n,this->_ico.getScale().y/n);
	}

	void update(int n)
	{
		this->_n=n;
	}

	void setColor(sf::Color color)
	{
		if(this->_textual)
		{
			this->_title_ui.setFillColor(color);
			this->_q_ui.setFillColor(color);
		}
		else
		{
			this->_q_ui.setFillColor(color);
		}
	}

	void setBlink(bool blink=true)
	{
		this->_blink=blink;
	}

	void hideValue(bool hide=true)
	{
		this->_hide=hide;
	}

	void draw(sf::RenderWindow &window)
	{
		if(!this->_textual)
		{
			window.draw(this->_ico);
			_update();

			if(this->_blink)
			{
				if(this->_chrono.getElapsedTime().asSeconds() > 1)
					window.draw(this->_q_ui);
				if(this->_chrono.getElapsedTime().asSeconds() > 2) this->_chrono.restart();
			}
			else
				if(!this->_hide)window.draw(this->_q_ui);
		}

		else
		{
			if(this->_blink)
			{
				if(this->_chrono.getElapsedTime().asSeconds() > 1)
				{
					window.draw(this->_title_ui);
					_update();
					window.draw(this->_q_ui);
				}
				if(this->_chrono.getElapsedTime().asSeconds() > 2 )this->_chrono.restart();

			}
			else
			{
				window.draw(this->_title_ui);
				_update();
				if(!this->_hide)window.draw(this->_q_ui);
			}
		}
	}
private:
sf::Sprite _ico;
sf::Text _title_ui;
sf::Text _q_ui;
sf::Clock _chrono;
std::string _text;
unsigned long long _n=0;
bool _textual=false;
bool _blink=false;
bool _hide=false;

void _update()
{
	if(!this->_textual) this->_q_ui.setString("x"+std::to_string(this->_n));
	else this->_q_ui.setString("  "+std::to_string(this->_n));

}

};
