/*
    Clay Shooter is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Clay Shooter is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Clay Shooter.  If not, see <http://www.gnu.org/licenses/>.

#-----------------------------------------------------------------------------------------#

 Author: Federico Salotti
 Organization/Company: Mr_Robot_Computers_Repair
 Country: Italy Barga(LU)
 release: 1.0 (08 March 2020 14:02)
 website: http://mrrobotcomputers.hopto.org

*/



class plateBattery
{
public:
	plateBattery(sf::Texture &texture, sf::Texture &texture2)
	{
		for(int i=0; i < PLATE_BATTERY; i++)
		{
			Plate plate(texture,texture2);
			this->_battery.push_back(plate);
		}
	}

	void newPlates(sf::Texture &texture,sf::Texture &texture2)
	{
		for(int i=0; i < PLATE_BATTERY; i++)
		{
			Plate plate(texture,texture2);
			this->_battery.push_back(plate);
		}
	}

	void preparePlate()
	{
		for(auto &plate : this->_battery)
			if(!plate.isReady())
			{
				plate.setReady();
				return;
			}
	}

	void check_collision(sf::Sprite view)
	{
		for(auto plate=this->_battery.begin(); plate != this->_battery.end(); ++plate)
			if((*plate).getGlobalBounds().intersects(view.getGlobalBounds()))
			{
				if((*plate).getDistance().y == 1)
					(*plate).broke();
				if((*plate).getDistance().y < 1)
					if((rand()%10+1)%2 == 0) (*plate).broke();

			}
	}

	void check_missing(ObjStatus &info)
	{
		for(auto plate=this->_battery.begin(); plate != this->_battery.end(); ++plate)
		{
			if((*plate).getXY().x < 0) {this->_battery.erase(plate);--plate;info.add();}
			if((*plate).getXY().x > 1000) {this->_battery.erase(plate);--plate;info.add();}
//			if((*plate).getXY().y < 0) {this->_battery.erase(plate);--plate;}
		}
	}

	void clean(ObjStatus &info)
	{
		for(auto plate=this->_battery.begin(); plate != this->_battery.end(); ++plate)
		{
			if((*plate).isBroke())
			{
				if((*plate).getDistance().y == 0.5) info.add(50);
				else if((*plate).getDistance().y == 0.25) info.add(75);
				else if((*plate).getDistance().y == 0.125) info.add(150);
				else
				{
					info.add(25);
					Particles particles((*plate).getXY().x,(*plate).getXY().y);
					this->_particles.push_back(particles);
				}
				this->_battery.erase(plate);
				--plate;
			}
		}
	}

	void assignPlatesSpawn()
	{
		for(auto &plate : this->_battery)
		{
			short pos_seed=rand()%100+1;
			if(pos_seed < 10)
				plate.setPosition(sf::Vector2f(200,300));
			else if(pos_seed > 10 && pos_seed < 20)
				plate.setPosition(sf::Vector2f(800,300));
			else if(pos_seed > 20 && pos_seed < 30)
				plate.setPosition(sf::Vector2f(10,50));
			else if(pos_seed > 30 && pos_seed < 40)
				plate.setPosition(sf::Vector2f(10,100));
			else if(pos_seed > 40 && pos_seed < 50)
				plate.setPosition(sf::Vector2f(980,100));
			else if(pos_seed > 50 && pos_seed < 60)
				plate.setPosition(sf::Vector2f(400,400));
			else
				plate.setPosition(sf::Vector2f(980,50));
		}
	}

	void assignPlatesMovement(int lv)
	{
		if(lv >= 4) lv=4;
		for(auto &plate : this->_battery)
		{
			if(plate.onRightHigh())
				plate.setMovement(-lv*2,0);

			if(plate.onLeftHigh())
				plate.setMovement(lv*2,0);

			if(plate.onLeftLow())
				plate.setMovement(lv*2,-lv*2);

			if(plate.onRightLow())
				plate.setMovement(-lv*2,-lv*2);

			if(plate.toHorizon())
				plate.setMovement(0,-lv*2);
		}
	}

	void movePlates()
	{
		for(auto &plate : this->_battery)
			plate.move();
	}

	void moveParticles()
	{
		_deleteParticles();
		for(auto &particle : this->_particles)
			particle.move();
	}

	void checkSteppingAway(ObjStatus &info)
	{
		for(auto plate=this->_battery.begin(); plate != this->_battery.end(); ++plate)
		{
			if((*plate).getXY().y < 21)
			{
				if((*plate).getDistance().y <= 0.125 && !(*plate).isBroke())
				{
					this->_battery.erase(plate);
					--plate;
					info.add();
				}

				else	(*plate).step_away();
			}
		}

	}

	void reset(sf::Texture &texture,sf::Texture &texture2,int lv)
	{
		this->_battery.clear();
		newPlates(texture,texture2);
		assignPlatesSpawn();
		assignPlatesMovement(lv);
	}

	void resetParticles()
	{
		this->_particles.clear();
	}

	int getPlatesNumber()
	{
		return this->_battery.size();
	}

	bool isEmpty()
	{
		if(this->_battery.size() == 0) return true;
		return false;
	}

	void draw(sf::RenderWindow &window)
	{
		for(auto &plate : this->_battery)
		{
			if(plate.isReady())
				plate.draw(window);
		}

		for(auto &particle : this->_particles)
			particle.draw(window);
	}
private:
std::vector<Plate> _battery;
std::vector <Particles> _particles;

void _deleteParticles()
{
	for(auto particle=this->_particles.begin(); particle != this->_particles.end(); ++particle)
	{
		if(particle->getSize() == 0)
		{
			this->_particles.erase(particle);
			--particle;
		}
	}
}

};
